package com.bitsi.core.bitsi_core

import android.util.Log
import com.bitsi.core.bitsi_core.BitsiCorePlugin.Companion.JITSI_PLUGIN_TAG
import io.flutter.plugin.common.EventChannel
import java.io.Serializable

/**
 * StreamHandler to listen to conference events and broadcast it back to Flutter
 */
class BitsiCoreEventStreamHandler private constructor(): EventChannel.StreamHandler, Serializable {
    companion object {
        val instance = BitsiCoreEventStreamHandler()
    }

    private var eventSink: EventChannel.EventSink? = null

    override fun onListen(arguments: Any?, eventSink: EventChannel.EventSink?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onListen")
        this.eventSink = eventSink
    }

    override fun onCancel(arguments: Any?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onCancel")
        eventSink = null
    }

    fun onOpened() {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onOpened")
        eventSink?.success(mapOf("event" to "opened"))
    }

    fun onClosed() {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onClosed")
        eventSink?.success(mapOf("event" to "closed"))
    }

    fun onConferenceWillJoin(data: MutableMap<String, Any>?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onConferenceWillJoin")
        eventSink?.success(mapOf("event" to "conferenceWillJoin", "data" to data))
    }

    fun onConferenceJoined(data: MutableMap<String, Any>?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onConferenceJoined")
        eventSink?.success(mapOf("event" to "conferenceJoined", "data" to data))
    }

    fun onConferenceTerminated(data: MutableMap<String, Any>?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onConferenceTerminated")
        eventSink?.success(mapOf("event" to "conferenceTerminated", "data" to data))
    }

    fun onAudioMutedChanged(data: MutableMap<String, Any>?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onAudioMutedChanged")
        eventSink?.success(mapOf("event" to "audioMutedChanged", "data" to data))
    }

    fun onParticipantJoined(data: MutableMap<String, Any>?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onParticipantJoined")
        eventSink?.success(mapOf("event" to "participantJoined", "data" to data))
    }

    fun onParticipantLeft(data: MutableMap<String, Any>?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onParticipantLeft")
        eventSink?.success(mapOf("event" to "participantLeft", "data" to data))
    }

    fun onEndpointTextMessageReceived(data: MutableMap<String, Any>?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onEndpointTextMessageReceived")
        eventSink?.success(mapOf("event" to "endpointTextMessageReceived", "data" to data))
    }

    fun onScreenShareToggled(data: MutableMap<String, Any>?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onScreenShareToggled")
        eventSink?.success(mapOf("event" to "screenShareToggled", "data" to data))
    }

    fun onParticipantsInfoRetrieved(data: MutableMap<String, Any>?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onParticipantsInfoRetrieved")
        eventSink?.success(mapOf("event" to "participantInfoRetrieved", "data" to data))
    }

    fun onChatMessageReceived(data: MutableMap<String, Any>?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onChatMessageReceived")
        eventSink?.success(mapOf("event" to "chatMessageReceived", "data" to data))
    }

    fun onChatToggled(data: MutableMap<String, Any>?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onChatToggled")
        eventSink?.success(mapOf("event" to "chatToggled", "data" to data))
    }

    fun onVideoMutedChanged(data: MutableMap<String, Any>?) {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onVideoMutedChanged")
        eventSink?.success(mapOf("event" to "videoMutedChanged", "data" to data))
    }

    fun onPictureInPictureWillEnter() {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onPictureInPictureWillEnter")
        eventSink?.success(mapOf("event" to "onPictureInPictureWillEnter"))
    }

    fun onPictureInPictureTerminated() {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onPictureInPictureTerminated")
        eventSink?.success(mapOf("event" to "onPictureInPictureTerminated"))
    }

    fun onReadyToClose() {
        Log.d(JITSI_PLUGIN_TAG, "BitsiCoreEventStreamHandler.onReadyToClose")
        eventSink?.success(mapOf("event" to "onReadyToClose"))
    }
}